/*
 * Copyright 2005 Hong Huang, Network Appliance, Inc.
 * Copyright 2005 Dan Gilson, Network Appliance, Inc.
 *
 * From: 
 * (C) Copyright 2000
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 */

#include <common.h>
#include "luna.h"
#include <asm/processor.h>
#include <spd_sdram.h>
#include <asm/io.h>
#include <command.h>

int board_pre_init (void)
{
   /*-------------------------------------------------------------------------+
   | Interrupt controller setup for the Luna board.
   | See Chapter 10
   |
   | Note: IRQ 0-15  405GP internally generated; active high; level sensitive
   |       IRQ 16    405GP internally generated; active low; level sensitive
   |       IRQ 17-24 RESERVED
   |       IRQ 25 (EXT IRQ 0) IRQ from MB Agent; active low; level sensitive
   |       IRQ 26 (EXT IRQ 1) IIRQ from MB Agent; active low; level sensitive
   |       IRQ 27 (EXT IRQ 2) Temp Warning; Active Low; Edge sensitive
   |       IRQ 28 (EXT IRQ 3) Temp Critical; Active Low; Edge sensitive
   |       IRQ 29 (EXT IRQ 4) Not Used
   |       IRQ 30 (EXT IRQ 5) Not Used
   |       IRQ 31 (EXT IRQ 6) Not Used
   |
   +-------------------------------------------------------------------------*/

	mtdcr (uicsr, 0xFFFFFFFF);	/* clear all ints */
	mtdcr (uicer, 0x00000000);	/* disable all ints */
	mtdcr (uiccr, 0x00000000);	/* set all to be non-critical */
	mtdcr (uicpr, 0xFFFFFF80);	/* set int polarities */
	mtdcr (uictr, 0x10000018);	/* set int trigger levels */
	mtdcr (uicvcr, 0x00000001);	/* set vect base=0,INT0 highest priority */
	mtdcr (uicsr, 0xFFFFFFFF);	/* clear all ints */

/*
 * after reset 
 * mtebc (pb0ap, 0x7F8FFE80); slowest settings
 * BME=0x0,TWT=0xFF,CSN=0x3,OEN=0x3,WBN=0x3,WBF=0x3,TH=0x7
 * 		RE=0x0,SOR=1,BEM=0,PEN=0
 * mtebc (pb0cr, 0xFFE28000); 2MB read-only bank. 
 * BAS=0xFFE,BS=0x1(2MB),BU=0x1(R/O),BW=0x0(8bit)
 *
 * banks 1-7 are zero'ed see table 8-3. DCR Contents After Reset
 */
	
#define mtebc(reg, data)  mtdcr(ebccfga,reg);mtdcr(ebccfgd,data)

/* Map in the CPLD for reset cause */

	/*mtebc (pb1ap, 0x02815480); original value from Walnut */
	/* This is from 16.6.3 Peripheral Ban Access Parameters
	 * BME=0x0,TWT=0x7,CSN=0x1,OEN=0x1,WBN=0x1,WBF=0x1,TH=0x7
	 * RE=0x0,SOR=1,BEM=0,PEN=0
	 */
	mtebc (pb1ap, 0x03855E80);
	/* This is from 16.6.2 Peripheral Bank Configuration Registers
	 * BAS=0xF00,BS=0x0(1MB),BU=0x1(R/W),BW=0x0( 8 bits) 
	 */
	mtebc (pb1cr, 0xF0008000);

	gpio_init();

	return 0;
}


/* ------------------------------------------------------------------------- */

/*
 * Check Board Identity:
 */

int checkboard (void)
{
	unsigned char *s = getenv ("serial#");
	unsigned char *e;

	puts ("Board: [");

	if (!s) {
		puts ("### No HW ID - assuming LUNA405");
	} else {
		for (e = s; *e; ++e) {
			if (*e == ' ')
				break;
		}
		for (; s < e; ++s) {
			putc (*s);
		}
	}
	puts ("]\n");

	return (0);
}


/* -------------------------------------------------------------------------
  initdram(int board_type) reads EEPROM via I2c. EEPROM contains all of
  the necessary info for SDRAM controller configuration
   ------------------------------------------------------------------------- */
long int initdram (int board_type)
{
/*  #define SDRAM_DEBUG */

	/* 405GP way of doing things */
	/*return  spd_sdram (0);*/

/*
 * For the luna board we have a known about of SDRAM.	
 * This is 64MB of ECC memory.
 *
 * There are no SPD (I2C) it is just glued down.
 * So,  configure it here and be done with it.
 *
 */
#define  SDRAM0_CFG_DCE          0x80000000
#define  SDRAM0_CFG_SRE          0x40000000
#define  SDRAM0_CFG_PME          0x20000000
#define  SDRAM0_CFG_MEMCHK       0x10000000
#define  SDRAM0_CFG_REGEN        0x08000000
#define  SDRAM0_CFG_ECCDD        0x00400000
#define  SDRAM0_CFG_EMDULR       0x00200000
#define  SDRAM0_CFG_DRW_SHIFT    (31-6)
#define  SDRAM0_CFG_BRPF_SHIFT   (31-8)

#define  SDRAM0_TR_CASL_SHIFT    (31-8)
#define  SDRAM0_TR_PTA_SHIFT     (31-13)
#define  SDRAM0_TR_CTP_SHIFT     (31-15)
#define  SDRAM0_TR_LDF_SHIFT     (31-17)
#define  SDRAM0_TR_RFTA_SHIFT    (31-29)
#define  SDRAM0_TR_RCD_SHIFT     (31-31)

#define MEMCLKOUT_2 1
#define MEMCLKOUT_3 2
#define MEMCLKOUT_4 3
#define RFTA_MEMCLKOUT_4 0
#define RFTA_5_MEMCLKOUT 1
#define RFTA_6_MEMCLKOUT 2
#define RFTA_7_MEMCLKOUT 3
#define RFTA_8_MEMCLKOUT 4
#define RFTA_9_MEMCLKOUT 5
#define RFTA_10_MEMCLKOUT 6

#define  SDRAM0_RTR_SHIFT        (31-15)
#define  SDRAM0_ECCCFG_SHIFT     (31-11)

/* SDRAM0_CFG enable macro  */
#define SDRAM0_CFG_BRPF(x) ( ( x & 0x3)<< SDRAM0_CFG_BRPF_SHIFT )

#define SDRAM0_BXCR_SZ_MASK  0x000e0000
#define SDRAM0_BXCR_AM_MASK  0x0000e000

#define SDRAM0_BXCR_SZ_SHIFT (31-14)
#define SDRAM0_BXCR_AM_SHIFT (31-18)

#define SDRAM0_BXCR_SZ(x)  ( (( x << SDRAM0_BXCR_SZ_SHIFT) & SDRAM0_BXCR_SZ_MASK) )
#define SDRAM0_BXCR_AM(x)  ( (( x << SDRAM0_BXCR_AM_SHIFT) & SDRAM0_BXCR_AM_MASK) )

/* Memory sizes */ 
#define _4M 0
#define _8M 1
#define _16M 2
#define _32M 3
#define _64M 4
#define _128M 5
#define _256M 6

#define MODE1 0
#define MODE2 1
#define MODE3 2
#define MODE4 3
#define MODE5 4
#define MODE6 5
#define MODE7 6

#define BE 1 /* Bank Enable */

#define sdram_HZ_to_ns(hertz) (1000000000/(hertz))
	
#define mem_mcopt1  0x20    /* memory controller options 1         */
#define mem_rtr     0x30    /* refresh timer reg           */
#define mem_pmit    0x34    /* power management idle timer         */
#define mem_mb0cf   0x40    /* memory bank 0 configuration         */
#define mem_mb1cf   0x44    /* memory bank 1 configuration         */
#define mem_mb2cf   0x48    /* memory bank 2 configuration         */
#define mem_mb3cf   0x4c    /* memory bank 3 configuration         */
#define mem_sdtr1   0x80    /* timing reg 1                */
	
#define MEGA_BYTE (1024*1024)

int total_size;

int sdram0_pmit=0x07c00000;
int sdram0_besr0=-1;
int sdram0_besr1=-1;
int sdram0_eccesr=-1;

int sdram0_ecccfg=0;
int sdram0_rtr=0x0c800000;
int sdram0_tr=0;

int sdram0_b0cr=0 | SDRAM0_BXCR_SZ( _64M ) | SDRAM0_BXCR_AM( MODE3 ) | BE;
int sdram0_b1cr=0; /* disabled */
int sdram0_b2cr=0; /* disabled */
int sdram0_b3cr=0; /* disabled */

int sdram0_cfg=0;

#if defined(CFG_ENABLE_ECC)
int ecc_on=1; 
#else
int ecc_on=0; 
#endif


/*------------------------------------------------------------------
    program all the registers.
 -------------------------------------------------------------------*/

#ifdef SDRAM_DEBUG
#define mtsdram0(reg, data)  printf(#reg " = 0x%x\n",data);\
			mtdcr(memcfga,reg);mtdcr(memcfgd,data)
#else
#define mtsdram0(reg, data)  mtdcr(memcfga,reg);mtdcr(memcfgd,data)
#endif

    /* disable memory controller so updates work */
    sdram0_cfg = 0;
    mtsdram0( mem_mcopt1, sdram0_cfg );

	mtsdram0( mem_besra , sdram0_besr0 ); /* Writing clears */
    mtsdram0( mem_besrb , sdram0_besr1 ); /* Writing clears */
    mtsdram0( mem_ecccf , sdram0_ecccfg ); /* disable all banks */
    mtsdram0( mem_eccerr, sdram0_eccesr ); /* Writing clears */
    mtsdram0( mem_rtr   , sdram0_rtr );
    mtsdram0( mem_pmit  , sdram0_pmit );
    mtsdram0( mem_mb0cf , sdram0_b0cr );
    mtsdram0( mem_mb1cf , sdram0_b1cr );
    mtsdram0( mem_mb2cf , sdram0_b2cr );
    mtsdram0( mem_mb3cf , sdram0_b3cr );

	/* set up CASL */
    sdram0_tr = MEMCLKOUT_3 << SDRAM0_TR_CASL_SHIFT;
    /* set up PTA */
    sdram0_tr |= MEMCLKOUT_3 << SDRAM0_TR_PTA_SHIFT;
    /* set up CTP */
    sdram0_tr |= MEMCLKOUT_4 << SDRAM0_TR_CTP_SHIFT;
    /* set LDF  */
    sdram0_tr |= MEMCLKOUT_3 << SDRAM0_TR_LDF_SHIFT;
    /* set RFTA */
    sdram0_tr |= RFTA_10_MEMCLKOUT << SDRAM0_TR_RFTA_SHIFT;
    /* set RCD */
    sdram0_tr |= MEMCLKOUT_2 << SDRAM0_TR_RCD_SHIFT ;
    mtsdram0( mem_sdtr1 , sdram0_tr );

	/* SDRAM have a power on delay,  500 micro should do */
	udelay(500);
	sdram0_cfg =  SDRAM0_CFG_DCE 
				| SDRAM0_CFG_BRPF(1) 
/*				| SDRAM0_CFG_ECCDD  */ /* allow ECC to be written */
				| SDRAM0_CFG_EMDULR;
	if(ecc_on)
		sdram0_cfg |= SDRAM0_CFG_MEMCHK;

	mtsdram0( mem_mcopt1, sdram0_cfg );
	
	total_size = 64 * MEGA_BYTE;	/* 64MB */

#if defined(CFG_ENABLE_ECC)
    {
        /* disable checking for all banks */
        unsigned long   *p;
#ifdef SDRAM_DEBUG
        puts ("disable ECC.. ");
#endif
		/* ECC was previously disabled */
		/* mtsdram0( mem_ecccf, 0x0 ); */  /* disable all banks */
        /* set up SDRAM Controller with ECC enabled */
#ifdef SDRAM_DEBUG
        puts ("setup SDRAM Controller.. ");
#endif
        udelay (600);
#ifdef SDRAM_DEBUG
        puts ("Zero fill the memory to set ECC..\n");
        puts ("B");
#endif
        /* now, fill all the memory */
        p = (unsigned long) 0;
        while ((unsigned long) p < total_size) {
            *p++ = 0L;
#ifdef SDRAM_DEBUG
            if (!((unsigned long) p % 0x00400000))  /* every 4MByte */
                puts ("*");
#endif
        }
        /* enable bank 0 */
#ifdef SDRAM_DEBUG
        puts ("E");
        puts ("enable ECC\n");
#endif
        udelay (400);
		mtsdram0( mem_ecccf, 0x00800000 ); /* enable bank 0 */
        udelay (400);
    }
#endif

	return (total_size);
}

/* ------------------------------------------------------------------------- */
static char * test_mode[] = {
	"data bus test",
	"address bus test",
	"stuck fault test",
	"pattern test", 
	"word walk",
	"byte walk",
	"partial word test",
	"byte pattern test",
	"ecc pattern test",
};

static void show_progress(void)
{
	printf(".");
	return;
}

static void print_mem_error(uint location, uint expect_data, 
						uint read_data, mem_test_suite_t mode)
{
	int cmp_data;
	
	cmp_data = expect_data ^ read_data;
	if (cmp_data & 0x0000ffff) {
		printf("MEM data compare error happened, check SDRAM U10\n");
	}
	if (cmp_data & 0xffff0000) {
		printf("MEM data compare error happened, check SDRAM U9\n");
	}

	printf("Error in memory test, %s\n", test_mode[mode]);
	printf("Location 0x%x, Expected Data 0x%x, Read Data 0x%x\n",
			location, expect_data, read_data);
}

static int check_ecc(uint ecc, int * ecc_p)
{
	int ecc_status;
	int retc;

	if ( ecc == 0) {
		return 0;
	}
		
	retc = 0;
	mtdcr(memcfga, mem_eccerr);
	ecc_status = mfdcr(memcfgd);
	if (ecc_status != 0) {	
		mtdcr(memcfga, mem_eccerr);
		mtdcr(memcfgd, ecc_status);
		retc = 1;
	}

	if ( ecc_status == 0) {
		return 0;
	}

	if (ecc_status & (ECC_BAD_U9)) {
		printf("Correctable ECC error happened, check SDRAM U9\n");
	}

	if (ecc_status & (ECC_BAD_U10)) {
		printf("Correctable ECC error happened, check SDRAM U10\n");
	}

	if (ecc_status & (ECC_BAD_U8)) {
		printf("ECC check bit error happened, check SDRAM U8\n");
	}	

	if (ecc_status & (ECC_UNCORR)) {
        printf("ECC uncorrectable error happened\n");
		*ecc_p = 1;
    }
	return retc ;
}
	
static int check_err_count (int *err_count_p)
{
	(*err_count_p)++;
	if (*err_count_p > 10) {
		return 1;
	}
	else {
		return 0;
	}
}
	
/***********************************************************************
 * mem_data_walk ()
 * Performs a walking 1 in a field of zeroes and a walking 0 in a
 * field of ones.  
 *
 * This test is intended to verify data lines within main memory
 * Expected to detect hard faults such as shorts and opens.
 **********************************************************************/
static int mem_data_walk(uint * mem_start, uint mem_size, uint ecc, int * ecc_p)
{
    uint * location;
    uint data;
    uint read_data;
	uint i;
	int err_count;
	uint * mem_test_start;

	show_progress();
	mem_test_start = mem_start + (mem_size/4)/64;
	err_count = 0;
	location = mem_test_start;
    data = 1;
    for (i = 0; i <32; i++) {
    	*location = data;
    	*mem_start = 0xffffffff;
		read_data = *location;
    	if (read_data != data) {
			*ecc_p = 1;
			print_mem_error((uint)location, data, read_data, DATA_WALK);
			if (check_err_count (&err_count)){
				return 1;
			}
		}
		if (check_ecc(ecc, ecc_p) != 0) {
			printf("addr = %p, data = 0x%x\n", location, data);
			if (check_err_count (&err_count)) {
				return 1;
			}
		}
		data <<= 1;
	}  

	data = 0xfffffffe;
	location = mem_test_start; 
	for (i = 0; i < 32; i++) {
		*location = data;
		*mem_start = 0;
		read_data = *location;
		if ( read_data != data) {
			*ecc_p = 1;
			print_mem_error((uint)location, data, read_data, DATA_WALK);
			if (check_err_count (&err_count)) {
				return 1;
			}
		}
		if ( check_ecc(ecc, ecc_p) != 0) {
			printf("addr = %p, data = 0x%x\n", location, data);
			if (check_err_count (&err_count)) {
				return 1;
			}
		}
		data <<= 1;
	}  
	return err_count;
}

/***********************************************************************
 * mem_addr_walk()
 *  Algorithm as follows:
 *  1) store log of address in each power-of-two address
 *  (place value into each byte so many bits get tested)
 *  2) check each power-of-two address for address pattern
 *
 * This test is intended to verify address lines within main memory
 * Expected to detect hard faults such as shorts and opens.
 **********************************************************************/
static int mem_addr_walk(uint * mem_start, uint mem_size, uint ecc, int * ecc_p)
{
    uint * location;
    int i;
    uint data;
    uint read_data;
	int err_count;

	show_progress();
	err_count = 0;
	data = 0x21426384;
	location = mem_start;
	i = 0x1;
	do {
		*location = data;
		location = mem_start + i;
		data += 0x01010101;
		i <<= 1;
	} while (i <= mem_size/4);

	location = mem_start;
	data = 0x21426384;
	i = 0x1;
	do {
		if ((read_data = *location ) != data) {
			*ecc_p = 1;
			print_mem_error((uint)location, data, read_data, ADDR_WALK);
			if (check_err_count (&err_count)) {
				return 1;
			}
		}
		if ( check_ecc(ecc, ecc_p) != 0) {
    		printf("addr = %p, data = 0x%x\n", location, data);
			if (check_err_count (&err_count)) {
 				return 1;
			}
		}

		data += 0x01010101;
		location = mem_start + i;
		i <<= 1;
	}while ( i <= mem_size/4);

	location = mem_start;
	data = 0x17263544;
	i = 0x1;
	do {
		*location = data;
		location = mem_start + i;
		data += 0x04040404;
		i <<= 1;
	} while (i <= mem_size/4);

	location = mem_start;
	data = 0x17263544;
	i = 0x1;
	do {
		if ((read_data = *location ) != data) {
			*ecc_p = 1;
			print_mem_error((uint)location, data, read_data, ADDR_WALK);
			if (check_err_count (&err_count)) {
				return 1;
			}
		}
		if ( check_ecc(ecc, ecc_p) != 0) {
    		printf("addr = %p, data = 0x%x\n", location, data);
			if (check_err_count (&err_count)) {
				return 1;
			}
		}

  		data += 0x04040404;
		location = mem_start + i;
		i <<= 1;
	}while ( i <= mem_size/4);

	location = mem_start + mem_size/4 -1;
	data = 0xa3b2c1d0;
	i = 0x1;
	do {
		*location = data;
		location = mem_start + mem_size/4 - 1 - i;
		data += 0x01010101;
		i <<= 1;
	} while ( i <= mem_size/4);

	location = mem_start + mem_size/4 - 1;
	data = 0xa3b2c1d0;
	i = 0x1;
	do {
		if ((read_data = *location) != data) {
			*ecc_p = 1;
			print_mem_error((uint)location, data, read_data, ADDR_WALK);
			if (check_err_count (&err_count)) {
				return 1;
			}
		}
		if ( check_ecc(ecc, ecc_p) != 0) {
    		printf("addr = %p, data = 0x%x\n", location, data);
			if (check_err_count (&err_count)) {
				return 1;
			}
		}

		data += 0x01010101;
		location = mem_start + mem_size/4 - 1 - i;
		i <<= 1;
	} while ( i <= mem_size/4);

	location = mem_start + mem_size/4 - 1;
	data = 0xb1a29384;
	i = 0x1;
	do {
		*location = data;
		location = mem_start + mem_size/4 - 1 - i;
		data += 0x02020202;
		i <<= 1;
	} while ( i <= mem_size/4);

	location = mem_start + mem_size/4 - 1;
	data = 0xb1a29384;
	i = 0x1;
	do {
		if ((read_data = *location) != data) {
			*ecc_p = 1;
			print_mem_error((uint)location, data, read_data, ADDR_WALK);
			if (check_err_count (&err_count)) {
				return 1;
			}
  		}
		if ( check_ecc(ecc, ecc_p) != 0) {
    		printf("addr = %p, data = 0x%x\n", location, data);
			if (check_err_count (&err_count)) {
				return 1;
			}	
		}

		data += 0x02020202;
  		location = mem_start + mem_size/4 - 1 - i;
		i <<= 1;
	} while ( i <= mem_size/4);
	return err_count;
}

/***********************************************************************
 * mem_stuck_faults()
 *  Algorithm as follows:
 *  1) store zero-bits to all of memory
 *  2) check for zero-bits in each location
 *  3) store one-bits in all of memory
 *  4) check for one-bits in each location
 *  5) store zero-bits to all of memory
 *  6) check for zero-bits in each location
 *
 * This test is intended to check each and every cell in DRAM for
 * stuck-at faults to ferret out gross errors quickly.  An extra pass
 * is used to verify that both 0->1 and 1->0 transitions are tested.
 **********************************************************************/
static int mem_stuck_fault_test(uint * mem_start, uint mem_size, uint ecc, int * ecc_p)
{
	uint * location;
	uint data;
	uint read_data;
	int err_count;

	show_progress();
	err_count = 0;
	location = mem_start;
	data = 0x00000000;
	while (location < ( mem_start + mem_size/4 )) {
		*location++ = data;
	}

	show_progress();
	location = mem_start;
	do {
		if ((read_data = *location ) != data) {
			*ecc_p = 1;
			print_mem_error((uint)location, data, read_data, STUCK_FAULT);
			if (check_err_count (&err_count)) {
				return 1;
			}
 		}
	}while (++location < (mem_start + mem_size/4));

	location = mem_start;
	data = 0xffffffff;

	show_progress();
	while (location < (mem_start + mem_size/4)) {
		*location++ = data;
	}

	show_progress();
	location = mem_start;
	do{
		if ((read_data = *location) != data) {
			*ecc_p = 1;
			print_mem_error((uint)location, data, read_data, STUCK_FAULT);
			if (check_err_count (&err_count)) {
				return 1;
			}
  		}
	}while (++location < (mem_start + mem_size/4));

	show_progress();
	location = mem_start;
	data = 0x00000000;
	while (location < (mem_start + mem_size/4)) {
		*location++ = data;
	}

	show_progress();
	location = mem_start;
	do {
		if ((read_data = *location ) != data) {
			*ecc_p = 1;
			print_mem_error((uint)location, data, read_data, STUCK_FAULT);
			if (check_err_count (&err_count)) {
				return 1;
			}
 		}
	}while (++location < (mem_start + mem_size/4));

	err_count += check_ecc(ecc, ecc_p);

	return err_count;
}

static uint pattern[]= {
	0xaaaaaaaa,
	0x55555555,
	0x6DB66DB6,
	0xB6DBB6DB,
	0xDB6DDB6D,
	0x33333333,
	0x66666666,
	0xCCCCCCCC,
	0x99999999,
	0x33333333};

static int mem_data_pattern(uint * mem_start, uint mem_size, uint ecc, int * ecc_p)
{
	uint * location;
	int num_patterns = sizeof(pattern)/sizeof(pattern[0]);
	uint data_read;
	uint i;
	int err_count;
	uint * mem_test_start;

	show_progress();
	mem_test_start = mem_start + (mem_size/4) /64 ;
	location = mem_test_start;
	err_count = 0;
	for (i = 0 ; i < num_patterns; i++) {
		*location = pattern[i];
		*mem_start = 0xffffffff;	
		data_read = *location;
		if (data_read != pattern[i]) {
			*ecc_p = 1;
			print_mem_error((uint)location, pattern[i], data_read,PATTERN_TEST);
			if (check_err_count (&err_count)) {
				return 1;
			}
		}
		if ( check_ecc(ecc, ecc_p) !=0 ){
    		printf("addr = %p, data = 0x%x\n", location, data_read);
			if (check_err_count (&err_count)) {
				return 1;
			}
		}
	}
	return err_count;
}		

/***********************************************************************
 * mem_word_walk ()
 * Performs a walking ffffffff word in a field of 64 words of zero and
 * a walking 00000000 word in a field of 64 words of ones.  
 *
 * This test is intended to verify switching and holding functions
 * within DRAM.
 **********************************************************************/
static int mem_word_walk(uint * mem_start, uint mem_size, uint ecc, int * ecc_p)
{
	uint * location, * base_location;
	uint i, n;
	uint exp_data;
	uint read_data;
	int err_count;
	uint * mem_test_start;

	show_progress();
	mem_test_start = mem_start + (mem_size/4) /64;
	err_count = 0;
	base_location = mem_start;
	location = mem_test_start;
	for (n = 0; n < 64; n++) {
		for (i = 0; i < 64; i++) {
			*(location + i) = 0x0;
			if (i == n) {
				*(location + i) = 0xffffffff;
			}
		}	
		for ( i = 0 ; i < 64; i++) {
			*(base_location + i) = 0x5A5A5A5A;
		}
		for (i = 0; i < 64; i++) {
			read_data = *(location + i); 
			exp_data = 0x0; 
			if ( i == n) {
				exp_data = 0xffffffff;	
			}
			if (read_data != exp_data) {
				*ecc_p = 1;
				print_mem_error((uint)(location+i), exp_data, read_data, WORD_WALK);
				if (check_err_count (&err_count)) {
					return 1;
				}
			}
			if (check_ecc(ecc, ecc_p) != 0) {
				printf("addr = %p, data = 0x%x\n", location+i, read_data);
				if (check_err_count (&err_count)) {
					return 1;
				}
			}
		}
	}
					
	for (n = 0; n < 64; n++) { 
		for (i = 0; i < 64; i++) { 
			*(location + i) = 0xffffffff; 
			if (i == n) {   
				*(location + i) = 0x0;
			}
		}
		for ( i = 0 ; i < 64; i++) {
			*(base_location + i) = 0xA5A5A5A5;
		}
		for (i = 0; i < 64; i++) {
			read_data = *(location + i); 
			exp_data = 0xffffffff; 
			if ( i == n) {
				exp_data = 0x0;  
			}
			if (read_data != exp_data) {
				*ecc_p = 1;
				print_mem_error((uint)(location+i), exp_data, read_data, WORD_WALK);
				if (check_err_count(&err_count)) {
					return 1;
				}
			}
			if ( check_ecc(ecc, ecc_p) != 0) {
				printf("addr = %p, data = 0x%x\n", location+i, read_data);
				if (check_err_count (&err_count)) {
					return 1;
				}
			}
		}
	}	
	return err_count;
}

/***********************************************************************
 * mem_byte_walk ()
 * Performs a walking ff byte in a field of 256 bytes of zero and
 * a walking 00 byte in a field of 256 bytes of ones.  
 * In addition to the walking zeroes and ones,a similar pair of tests
 * is run with a complement byte stored at every *third* byte.
 *
 * This test is intended to check PC memory ability to handle partial
 * word operations. This is a check primarily of the control logic to deal 
 * with partial word operations to/from memory.
 **********************************************************************/
static int mem_byte_walk(uint * mem_start, uint mem_size, uint ecc, int * ecc_p)
{
	unsigned char * location, * base_location;
	int i, n;
	uint exp_data;
	uint read_data;
	uint * wordloc;
	int err_count;
	uint * mem_test_start;

	show_progress();
	err_count = 0;
	mem_test_start = mem_start + (mem_size/4)/64;
	base_location = (unsigned char *)mem_start;
	location = (unsigned char *)mem_test_start;
	for (n = 0; n < 256; n++) {
		for (i = 0; i < 256; i++) {
			*(location + i) = 0x00;
			if (i == n ) {
				*(location + i) = 0xff;
			}
		}
		for (i = 0; i < 256; i++) {
			*(base_location +i) = 0x5A;
		}
		for ( i = 255; i >= 0; i--) {
			read_data = *(location +i);
			exp_data = 0x00;
			if (i == n) {
				exp_data = 0xff;
			}
			if (read_data != exp_data) {
				*ecc_p = 1;
				print_mem_error((uint)(location+i), exp_data, read_data, BYTE_WALK);
				if (check_err_count(&err_count) != 0) {
					return 1;
				}
			}
			if ( check_ecc(ecc, ecc_p) != 0) {
				printf("addr = %p, data = 0x%x\n", location+i, read_data);
				if (check_err_count(&err_count) != 0) {
					return 1;
				}
			}
		}
	}

	for (n = 0; n < 256; n++) {
   		for (i = 255; i >= 0; i--) {
			*(location + i) = 0xff;
			if (i == n ) {
				*(location + i) = 0x00;
			}
		}
		for (i = 0; i < 256; i++) {
		*(base_location +i) = 0xA5;
		}
		for ( i = 0; i < 256; i++) {
			read_data = *(location +i);
			exp_data = 0xff;
			if (i == n) {
				exp_data = 0x00;
			}
			if (read_data != exp_data) {
				*ecc_p = 1;
				print_mem_error((uint)(location+i), exp_data, read_data,BYTE_WALK);
				if (check_err_count(&err_count) != 0) {
					return 1;
				}
			}
			if (check_ecc(ecc, ecc_p) !=0 ) {
				printf("addr = %p, data = 0x%x\n", location+i, read_data);
				if (check_err_count(&err_count) != 0) {
					return 1;
				}
			}
		}
	}

	for (n = 0; n < 256; n++) {
		for (i = 0; i < 256; i++) {
			*(location + i) = 0x00;
			if (i == n ) {
				*(location + i) = 0xfd;
			}
		}
		for (i = 0; i < 256; i++) {
			*(base_location +i) = 0x5A;
		}
		for ( i = 255; i >= 0; i--) {
			read_data = *(location +i);
			exp_data = 0x00;
			if (i == n) {
				exp_data = 0xfd;
			}
			if (read_data != exp_data) {
				*ecc_p = 1;
				print_mem_error((uint)(location+i), exp_data, read_data, BYTE_WALK);
				if (check_err_count(&err_count) != 0 ) {
					return 1;
				}
			}
			if (check_ecc(ecc, ecc_p) != 0) {
				printf("addr = %p, data = 0x%x\n", location+i, read_data);
				if (check_err_count(&err_count) != 0) {
					return 1;
				}
			}
		}
	}

	for (n = 0; n < 256; n++) {
		for (i = 255; i >= 0; i--) {
			*(location + i) = 0xfd;
			if (i == n ) {
				*(location + i) = 0x00;
			}
		}
		for (i = 0; i < 256; i++) {
 			*(base_location +i) = 0xA5;
		}
		for ( i = 0; i < 256; i++) {
			read_data = *(location +i);
			exp_data = 0xfd;
			if (i == n) {
				exp_data = 0x00;
			}
			if (read_data != exp_data) {
				*ecc_p = 1;
				print_mem_error((uint)(location+i), exp_data, read_data, BYTE_WALK);
				if (check_err_count(&err_count) != 0) {
					return 1;
				}
			}
			if (check_ecc(ecc, ecc_p) != 0) {
				printf("addr = %p, data = 0x%x\n", location+i, read_data);
				if (check_err_count(&err_count) != 0) {
					return 1;
				}
			}
		}
	}

	wordloc = (uint *)location;
	for (n = 0; n < 256; n++) {
		*wordloc++ = 0x00000000;
		*((uint *)(base_location) + n) = 0x12345678;
	}
	for (i = 0; i < 1024; i+=3) {
		*(location +i) = 0xff;
		*(base_location + i) = 0x5A;
	}
	for (i = 0; i < 1024; i++) {
		read_data = *(location + i);
		if ( i%3) {
			exp_data = 0x00;
		} else {
			exp_data = 0xff;
		}
		if (read_data != exp_data) {
			*ecc_p = 1;
			print_mem_error((uint)(location+i), exp_data, read_data, BYTE_WALK);
			if (check_err_count (&err_count) != 0) {
				return 1;
			}
		}
		if (check_ecc(ecc, ecc_p) != 0) {
			printf("addr = %p, data = 0x%x\n", location+i, read_data);
			if (check_err_count(&err_count) != 0) {
				return 1;
			}
		}
	}

	wordloc = (uint *)location;
	for (n = 0; n < 256; n++) {
		*wordloc++ = 0x00000000;   
		*((uint *)(base_location) + n) = 0x12345678;  
	}
	for (i = 0; i < 1024; i+=3) {  
		*(location +i) = 0xdf; 
		*(base_location + i) = 0x5A;   
	}
	for (i = 0; i < 1024; i++) {   
		read_data = *(location + i);   
		if ( i%3) {    
			exp_data = 0x00;   
		} else {   
			exp_data = 0xdf;   
		}
		if (read_data != exp_data) {   
			*ecc_p = 1;
			print_mem_error((uint)(location+i), exp_data, read_data, BYTE_WALK);
			if (check_err_count(&err_count) != 0) {
 				return 1;  
			}
		}
		if (check_ecc(ecc, ecc_p) != 0) {
			printf("addr = %p, data = 0x%x\n", location+i, read_data);
			if (check_err_count(&err_count) != 0) {
				return 1;
			}
		}
	}

	return err_count;
}	
	
/***********************************************************************
 * mem_partial ()
 * Another test of partial-word stores and loads, this test does a
 * series of byte and short stores interspersed with word stores,
 * followed by a series of loads and compares.
 *
 * This is intended to verify correct behavior of partial word merge
 * logic and to check PC memory ability to handle partial word
 * operations .  This is a check primarily of the control logic to deal
 * with partial word operations to/from memory.
 **********************************************************************/
static int mem_partial(uint * mem_start, uint mem_size, uint ecc, int * ecc_p)
{
	uint i, n;
	unsigned char * location, * baseloc;
	uint expected;
	uint * mem_test_start;
	int err_count;
	typedef struct{
		unsigned short s1;
		unsigned short s2;
		unsigned long  w1;
		unsigned long  w2;
		unsigned char  b1;
		unsigned char  b2;
		unsigned short s3;
		unsigned long  w3;
		unsigned long  w4;
	}partial;
	
	partial * pp, *bb;

	show_progress();
	err_count = 0;
	mem_test_start = mem_start + (mem_size/4)/64;

	baseloc = (unsigned char *)mem_start;
	bb = (partial *)baseloc;
	for (i = 0; i<500; i++) {
		bb->s1 = 0x1aa1;
		bb->s2 = 0x2aa2;
		bb->w1 = 0xfffffff1;
		bb->w2 = i;
		bb->b1 = 0xb1;
		bb->b2 = 0xb2;
		bb->s3 = 0x3aa3;
		bb->w3 = i;
		bb->w4 = 0xfffffff4;
		bb++;
	}

	bb = (partial *)baseloc;
	location = (unsigned char *) mem_test_start;
	n = 0;
	pp = (partial *)location;
	for ( i = 0; i<500; i++) {
		pp->s2 = 0xbabe;
		pp->w1 = i;
		pp->w2 = 0xfacade12;
		pp->b2 = 0xbb;
		pp->w3 = 0xefface89;
    	pp->w4 = i;
    	pp->s3 = 0xdead;
    	pp->b1 = 0x66;
    	pp++;
		n = bb->w3;
		bb++;
	}
	pp = (partial *)location;
	for (i = 0; i < 500; i++) {
		if (pp->s2 != 0xbabe) {
			*ecc_p = 1;
			expected = 0xbabe;
			print_mem_error((uint)location, expected, pp->w1,PARTIAL_WORD);
			if (check_err_count(&err_count) != 0) {
   				return 1;
			}
		}
		if (pp->w1 != i) {
			*ecc_p = 1;
			expected = i;
			print_mem_error((uint)location, expected, pp->w1,PARTIAL_WORD);
			if (check_err_count(&err_count) != 0) {
				return 1;
			}
		}
		if (pp->w2 != 0xfacade12) {
			*ecc_p = 1;
			expected = 0xfacade12;
			print_mem_error((uint)location, expected, pp->w2,PARTIAL_WORD);
			if (check_err_count(&err_count) != 0) {
				return 1;
			}
		}
		if (pp->b2 != 0xbb){
			*ecc_p = 1;
			expected = 0xbb;
			print_mem_error((uint)location, expected, pp->b2,PARTIAL_WORD);
			if (check_err_count(&err_count) != 0) {
				return 1;
			}
		}
    	if (pp->w3 != 0xefface89) {
			*ecc_p = 1;
			expected = 0xefface89;
			print_mem_error((uint)location, expected, pp->w3, PARTIAL_WORD);
			if (check_err_count(&err_count) != 0) {
 				return 1;
			}
		}
    	if (pp->w4 != i){
			*ecc_p = 1;
			expected = i;
			print_mem_error((uint)location, expected, pp->w4, PARTIAL_WORD);
			if (check_err_count(&err_count) != 0) {
				return 1;
			}
		}
		if (pp->s3 != 0xdead) {
			*ecc_p = 1;
			expected = 0xdead;
			print_mem_error((uint)location, expected, pp->s3, PARTIAL_WORD);
			if (check_err_count(&err_count) != 0) {
				return 1;
			}
		}
    	if (pp->b1 != 0x66) {
			*ecc_p = 1;
			expected = 0x66;
			print_mem_error((uint)location, expected, pp->b1, PARTIAL_WORD);
			if (check_err_count(&err_count) != 0) {
 				return 1;
 			}

		}
		if (check_ecc(ecc, ecc_p) != 0) {
			if (check_err_count(&err_count) != 0) {
				return 1;
			}	
		}
		pp++;
	}
	return err_count;
}
	
/***********************************************************************
 * mem_byte_pat ()
 * Spins through all 256 patterns in each byte of a word at each
 * 2MB "bank" boundary, testing one byte at a time.  
 *
 * This test is intended to verify parity generation/check logic
 **********************************************************************/
static int mem_byte_pat(uint * mem_start, uint mem_size, uint ecc, int * ecc_p)
{
	uint i, tmp;
	uint *location;
	uint read_data, expect_data;
	uint n;
	int err_count;
	uint * mem_test_start;
				
	show_progress();
	tmp = 0;
	err_count = 0;
	mem_test_start = mem_start + (mem_size/4)/64;
	location = mem_test_start;
	n = 2;
	do {
		for (i = 0; i < 256; i++) {
			expect_data = i << 24;
			*location = expect_data;
			tmp |= *mem_start;
			read_data = *location;
			if (read_data != expect_data) {
				*ecc_p = 1;
				print_mem_error((uint)location, expect_data, read_data, BYTE_PATTERN);
				if (check_err_count(&err_count) != 0) {
					return 1;
				}
			}
			if (check_ecc(ecc, ecc_p) != 0) {
				if (check_err_count(&err_count) != 0) {
					return 1;
				}
			}
		}
		for (i = 0; i < 256; i++) {
			expect_data = i << 16;
			*location = expect_data;
			tmp |= *mem_start;
			read_data = *location;
			if (read_data != expect_data) {
				*ecc_p = 1;
				print_mem_error((uint)location, expect_data, read_data, BYTE_PATTERN);
				if (check_err_count(&err_count) != 0) {
					return 1;
				}
			}
			if (check_ecc(ecc, ecc_p) != 0) {
				if (check_err_count(&err_count) != 0) {
 					return 1;
				}
			}
		}
		for (i = 0; i < 256; i++) {
			expect_data = i << 8;
			*location = expect_data;
			tmp |= *mem_start;   
			read_data = *location;
			if (read_data != expect_data) {
				*ecc_p = 1;
				print_mem_error((uint)location, expect_data, read_data, BYTE_PATTERN);
				if (check_err_count(&err_count) != 0) {
 					return 1;
				}
			}
			if (check_ecc(ecc, ecc_p) != 0) {
 				if (check_err_count(&err_count) != 0) {
					return 1;
				}
			}
		}
		for (i = 0; i < 256; i++) {
 			expect_data = i;
			*location = expect_data;
			tmp |= *mem_start;   
			read_data = *location;
			if (read_data != expect_data) {
				*ecc_p = 1;
				print_mem_error((uint)location, expect_data, read_data, BYTE_PATTERN);
				if (check_err_count(&err_count) != 0) {
  					return 1;
				}
			}
			if (check_ecc(ecc, ecc_p) != 0) {
 				if (check_err_count(&err_count) != 0) {
					return 1;
				}
			}
		}
		location++;
	}while (--n);
	return err_count;
}

static uint ecc_pattern_even_1[]= {
/* for even word, these data will generate ecc pattern */
    0x00000000,  /* 0x00 */
    0xc0008000,  /* 0x01 */
    0xc0000080,  /* 0x02 */
    0xc0000800,  /* 0x04 */
    0x40009000,  /* 0x08 */
    0x20003000,  /* 0x10 */
    0x40003000,  /* 0x20 */
    0x80003000,  /* 0x40 */
    0x90001000,  /* 0x80 */
};

static uint ecc_pattern_even_0[] = {
	0x60000001,  /* 0XFE */
	0x60000100,  /* 0xFD */
	0x60000010,  /* 0xFB */
	0xc0020000,  /* 0xF7 */
	0xc0010000,  /* 0xEF */
	0xa0010000,  /* 0xDF */
	0x60010000,  /* 0xBF */
	0x60000002,  /* 0x7F */
	0xc0004001  /* 0xFF */
};

static uint ecc_pattern_odd_1[] = {
/* for odd word, these data will generate ecc pattern */
	0x00000000,  /* 0x00 */
	0x80400008,  /* 0x01 */
	0xc0000008,  /* 0x02 */
	0x80040008,  /* 0x04 */
	0x84000200,  /* 0x08 */
	0x88000200,  /* 0x10 */
	0x88000400,  /* 0x20 */
	0x88000800,  /* 0x40 */
	0x88000100,  /* 0x80 */
};

static uint ecc_pattern_odd_0[] = {
	0x21000001,  /* 0xfe */
	0x20010001,  /* 0xfd */
	0x30000001,  /* 0xfb */
	0xa0010000,  /* 0xf7 */
	0x80000804,  /* 0xef */
	0xa0008000,  /* 0xdf */
	0xa0004000,  /* 0xbf */
	0x90004000,  /* 0x7f */
	0xa0000840   /* 0xff */
    };

/*************************************************************************
 * This test will test the ECC chip by generating walking '0' and 
 * walking '1' pattern on the data bus off ECC chip, both even word and 
 * odd word will be tested
 ************************************************************************/
static int ecc_pat_test(uint * mem_start, uint mem_size, int * ecc_p)
{
	uint * location;
    uint read_data_even, read_data_odd;
    uint i;
    int err_count;
	uint * mem_test_start;

	show_progress();
	err_count = 0;
	mem_test_start = mem_start + (mem_size/4)/64;
	location = mem_test_start ;
	for ( i = 0; i < 9; i++) {
		*location = ecc_pattern_even_1[i];
		*(location+1) = ecc_pattern_odd_1[i];
		*mem_start = 0xffffffff;
		read_data_even = *location;
		read_data_odd = *(location + 1);
		if ( (read_data_even != ecc_pattern_even_1[i] ) ||
				(read_data_odd != ecc_pattern_odd_1[i]) ) {
			*ecc_p = 1;
			print_mem_error((uint)location, ecc_pattern_even_1[i], 
								read_data_even, ECC_PATTERN);
			print_mem_error((uint)(location + 1), ecc_pattern_odd_1[i], 
								read_data_odd, ECC_PATTERN);
			if (check_err_count (&err_count)) {
				return 1;
			}
		}
		if (check_ecc(1, ecc_p) != 0) {
			printf("addr = %p, data = 0x%x\n", location, read_data_even);
			printf("addr = %p, data = 0x%x\n", location + 1, read_data_odd);
			if (check_err_count (&err_count)) {
				return 1;
			}
		}

	}
			
	for ( i = 0; i < 9; i++) {
		*location = ecc_pattern_even_0[i];
		*(location+1) = ecc_pattern_odd_0[i];
		*mem_start = 0xffffffff;
		read_data_even = *location;
		read_data_odd = *(location + 1);
		if ( (read_data_even != ecc_pattern_even_0[i] ) ||
				(read_data_odd != ecc_pattern_odd_0[i]) ) {
			*ecc_p = 1;
			print_mem_error((uint)location, ecc_pattern_even_0[i],
							read_data_even, ECC_PATTERN);
			print_mem_error((uint)(location + 1), ecc_pattern_odd_0[i],
							read_data_odd, ECC_PATTERN);
			if (check_err_count (&err_count)) {
				return 1;
			}
		}
		if (check_ecc(1, ecc_p) != 0) {
			printf("addr = %p, data = 0x%x\n", location, read_data_even);
			printf("addr = %p, data = 0x%x\n", location + 1, read_data_odd);
			if (check_err_count (&err_count)) {
				return 1;
			}
		}
	}
	
	return err_count;
}

/***********************************************************************
 * This test will write data to the location, read back, modify one bit,
 * then write back with ECC generation detected.  The enable ECC 
 * correction, read back data, there shuould be ECC correction happened, 
 * and the location and status should be logged in the ECC status register 
 ***********************************************************************/ 
static int ecc_injection_test(uint * mem_start, uint mem_size, int *ecc_p)
{
	int sdram0_cfg;
	uint data, temp1, temp2, save_data;
	int ecc_status;
    int retc;
	uint * mem_test_start;

#if defined(CFG_ENABLE_ECC)
int ecc_on=1;
#else
int ecc_on=0;
#endif

	retc = 0;
	mem_test_start = mem_start + (mem_size/4)/64;

	show_progress();
	/* disable ECC generation, and the write to the ecc chip */
	udelay(500);
    sdram0_cfg =  SDRAM0_CFG_DCE
				| SDRAM0_CFG_BRPF(1)
				| SDRAM0_CFG_ECCDD
				| SDRAM0_CFG_EMDULR;

	mtsdram0( mem_mcopt1, sdram0_cfg );
	udelay(500);
	
	/* read original good data */
	data = *(mem_test_start);
	temp1 = data;
	temp2 = data;
	save_data = data;

	/* modify one bit */
	temp1 &= 0xfffffffe;
	temp2 &= 0x1;

	data = temp1 | (~temp2 & 0x1);

	/* write back with ECC generation off */
	*(mem_test_start) = data;

	/* enable ECC generation */
	udelay(500);
	sdram0_cfg =  SDRAM0_CFG_DCE
				| SDRAM0_CFG_BRPF(1)
				| SDRAM0_CFG_EMDULR;
	if(ecc_on)
		sdram0_cfg |= SDRAM0_CFG_MEMCHK;
	
	mtsdram0( mem_mcopt1, sdram0_cfg );	
	udelay(5000);

	/* read back modified data, ECC correction should happen */ 
	data = *(mem_test_start);

	mtdcr(memcfga, mem_eccerr);
	ecc_status = mfdcr(memcfgd);

	*(mem_test_start) =  save_data;
	mtdcr(memcfga, mem_eccerr);
	mtdcr(memcfgd, ecc_status);


	if ( (data != save_data) | (ecc_status == 0)) {
		printf("ECC correction didn't happen, check the processor\n");
		retc = 1;
	}

	if ( !(ecc_status & (ECC_BAD_U10)) || !(ecc_status & ECC_CORR) ) {
		printf("ECC status report not correct, check processor\n");
		retc = 1;
	}

	check_ecc(1, ecc_p);

	return retc;
}
		
int testdram (void)
{
	DECLARE_GLOBAL_DATA_PTR; 
	/* TODO: XXX XXX XXX */
    uint * mem_start = CFG_SDRAM_BASE;
	uint mem_size = gd->ram_size;
	volatile __u32	value;
	uint rc;
	__u8 data;
	int ecc_ucerror;

    /* data = in8(CPLD_PHYS_ADDR); */
    data = *((__u8 *) CPLD_PHYS_ADDR) & 0xF0 ; /* we only want the top mibble */
	debug ("testdram: CPLD value is 0x%X\n", data );
	debug ("testdram: memcheck is %s\n", getenv("memcheck") ); 
	ecc_ucerror = 0;

	/* only do testing if we have powered up OR environment var says so */
	if (    (data == REBOOTED_VIA_POWER_ON) 
	 	 || (data == (EXEC_BACKUP_FW | REBOOTED_VIA_POWER_ON))
		 || (strcmp(getenv("memcheck"), "yes") == 0)
		)
	{
		rc = 0;
#if 0
		printf("test with ECC off......\n");
		udelay(500);
    	sdram0_cfg =  SDRAM0_CFG_DCE
                	| SDRAM0_CFG_BRPF(1)
                	| SDRAM0_CFG_EMDULR;

    	mtsdram0( mem_mcopt1, sdram0_cfg );
    	udelay(500);

		rc += mem_data_walk(mem_start, mem_size, 0);

        rc += mem_addr_walk(mem_start, mem_size, 0);
		
		rc += mem_stuck_fault_test(mem_start, mem_size, 0);

        rc += mem_data_pattern(mem_start, mem_size, 0);

        rc += mem_word_walk(mem_start, mem_size, 0);

        rc += mem_byte_walk(mem_start, mem_size, 0);

        rc += mem_partial(mem_start, mem_size, 0);

        rc += mem_byte_pat(mem_start, mem_size, 0);

		if ( rc == 0) {
			printf("test with ECC off PASS\n");
		}else {
			printf("test with ECC off FAIL\n");
		}

		/* enable ECC generation */
    	udelay(500);
    	sdram0_cfg =  SDRAM0_CFG_DCE
                	| SDRAM0_CFG_BRPF(1)
                	| SDRAM0_CFG_EMDULR;
    	if(ecc_on)
        	sdram0_cfg |= SDRAM0_CFG_MEMCHK;

    	mtsdram0( mem_mcopt1, sdram0_cfg );
    	udelay(5000000);
#endif
		rc += mem_data_walk(mem_start, mem_size, 1, &ecc_ucerror);	

		rc += mem_addr_walk(mem_start, mem_size, 1, &ecc_ucerror);

		rc += mem_stuck_fault_test(mem_start, mem_size, 1, &ecc_ucerror);

		rc += mem_data_pattern(mem_start, mem_size, 1, &ecc_ucerror);

		rc += mem_word_walk(mem_start, mem_size, 1, &ecc_ucerror);

		rc += mem_byte_walk(mem_start, mem_size, 1, &ecc_ucerror);

		rc += mem_partial(mem_start, mem_size, 1, &ecc_ucerror);

		rc += mem_byte_pat(mem_start, mem_size, 1, &ecc_ucerror);

		rc += ecc_pat_test(mem_start, mem_size, &ecc_ucerror); 

		rc += ecc_injection_test(mem_start, mem_size, &ecc_ucerror);

		printf("dram test ");	
		if (rc) {
			printf("FAIL\n");
		}else {
			printf("PASS\n");
		}
	
		/* only loop here in the manufacture fixture */
		/* will add gpio test pin later              */
		luna_gpio_get(LUNA_GPIO_TEST_MODE, &value);
		if ( value == 0 ) {
			if (rc || ecc_ucerror) {
				luna_gpio_clr(12);
				while(1);
			}
		} else {
			if ( rc ) {
				if ( ecc_ucerror ) {
					luna_gpio_clr(12);
                	while(1);
				}
			}
		}

	}
	return (0);
}

/* ------------------------------------------------------------------------- */


/* IBM 405GP DCRs */
#define CPC0_CR0        0xb1        /* Chip control register 0 */

int gpio_init(void)
{
	__u32 cpc0_cr0;
	
	cpc0_cr0 = mfdcr(CPC0_CR0);    	/* get chip ctrl register */
	cpc0_cr0 |= 0x03F0E000;			/* OR in the bits that we want */
	mtdcr(CPC0_CR0, cpc0_cr0); 		/* write it back out */
	luna_gpio_open_drain(0x7fffff00,0x0);
	luna_gpio_tristate(0x7fffff00,0x0);
	/* Hold LUNA_GPIO_RESET_CPU high because */
	/* 		it is an active low signal that resets the CPU thru the CPLD */
	/* turn on LUNA_GPIO_STATUS_LED and LUNA_GPIO_POST_LED_7 */
	/* LUNA_GPIO_STATUS_LED outputs a lead on the edge connector that says RLM lives */
	/* LUNA_GPIO_POST_LED_7 enables the watchdog timer */
	/* luna_gpio_out(0x7fffff00,0x10080000);  */
	luna_gpio_out(0x7fffff00,0x28080000); 
	luna_gpio_tristate(0x7fffff00,0x3DDF8300);
	return 0;
}

int luna_gpio_tristate(__u32 mask, __u32 data)
{
	__u32 temp;

	temp = in32(GPIO0_TCR);
	temp = (temp & ~mask) | (data & mask);
	out32(GPIO0_TCR, temp);
    return 0;
}


int luna_gpio_open_drain(__u32 mask, __u32 data)
{
	__u32 temp;

	temp = in32(GPIO0_ODR);
	temp = (temp & ~mask) | (data & mask);
	out32(GPIO0_ODR, temp);
    return 0;
}

void print_luna_gpio_in(void)
{
	__u32 temp;
	int my_ret;

	my_ret = luna_gpio_in(0x7fffffff,&temp);

	printf("luna_gpio_in = 0x%lx\n",temp);
}

int luna_gpio_in(__u32 mask, volatile __u32 *data)
{
	__u32 temp;

	temp = in32(GPIO0_TCR);
	temp = (temp & ~mask);
	out32(GPIO0_TCR, temp);
    eieio();

    /*
    ** If the previous state was OUT, and GPIO0_IR is read once, then the
    ** data that was being OUTput will be read.  One way to get the right
    ** data is to read GPIO0_IR twice.
    */

    *data = in32(GPIO0_IR);
    *data = in32(GPIO0_IR) & mask;
    return 0;
}

int luna_gpio_out(__u32 mask, __u32 data)
{
	__u32 temp;

	temp = in32(GPIO0_OR);
	temp = (temp & ~mask) | (data & mask);
	out32(GPIO0_OR, temp);

    eieio();

	/* Table 23-1 says that 
	 * TS 0 = High Impedance 
	 * TS 1 = Active (drive GPIO_Out Signal) 
	 * Since we have already set the correct TS bits we should not have
	 * to do them each time. 
	 */
#if 0	
	/* Switch the pin back to high impedance state */
	temp = in32(GPIO0_TCR);
	temp |=  mask;
	out32(GPIO0_TCR, temp);
    eieio();
#endif

    return 0;
}

int luna_post_led(__u8 post_code)
{
	/* gpio_map from LSB to MSB of post_code */
	int gpio_map[] = 
		{(31-11), (31-22), (31-16), (31-15), (31-14), (31-13), (31-12), (31-5)};
	int i;

	for (i=0; i<8; i++) {
		luna_gpio_out( (1 << gpio_map[i])
					 , ( ((post_code >> i) & 1 ) << gpio_map[i]) );
	}
	
	return 0;
}

int luna_gpio_set(luna_gpio_t bit)
{
	return luna_gpio_out( 1 << (8 * sizeof(u32) - (bit + 1)), ~0);
}

int luna_gpio_get(luna_gpio_t bit, volatile __u32 *val)
{
	return luna_gpio_in( 1 << (8 * sizeof(u32) - (bit + 1)), val);
}

int luna_gpio_clr(luna_gpio_t bit)
{
	return luna_gpio_out( 1 << (8 * sizeof(u32) - (bit + 1)), 0);
}


int do_reset (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	luna_gpio_clr(LUNA_GPIO_RESET_CPU);
	while (1) ; /* wait forever */
	return 1;
}

int do_wdt(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	if (argc > 3) {
		printf ("Usage:\n%s\n",  cmdtp->usage);
	    return 1;
	}
	if (argc == 2) {
		printf ("wdt: setting gpio\n");
		luna_gpio_set(7);
	} else {
		printf ("wdt: clearing gpio\n");
		luna_gpio_clr(7);
	}
	return 0;
}
U_BOOT_CMD(
		wdt, CFG_MAXARGS, 0, do_wdt,
		"wdt - set the watchdog gpio\n",
		"   any param sets and no param clears"
		);
	
int stop_dog(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	if (argc > 3) {
		printf ("Usage:\n%s\n",  cmdtp->usage);
	    return 1;
	}
	printf ("stop_dog: must reboot to restart\n");
	luna_gpio_clr(12);
	return 0;
}
U_BOOT_CMD(
		stop_dog, CFG_MAXARGS, 0, stop_dog,
		"stop_dog - disables the watchdog\n",
		""
		);

static char * mygets(char *s, int len)
{
	int i;
	int c;

	for (i = 0; i < len; i++) {
		c = getc();
		putc(c);
		if ( c == 0x0d) {
			*s = '\0';
			break;
		}
		*s = c;
		s++;
	}

//	*s = '\0';
	return s;
}

int fvt(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	char * frup[4];
	char env_para[4][20];
	int i;
	char ch[8] ;

	frup[0] = getenv("ethaddr");
	frup[1] = getenv("serial#");
	frup[2] = getenv("part#");
	frup[3] = getenv("rev#");

	printf("We will now program all information for this RLM board.\n");
    printf("Please enter or scan in all the information requested.\n");
    printf("You will be given a chance to correct your entries or cancel\n");
    printf("this operation later.\n");

	printf("\n\n");
    printf("RLM board:\n");
	
	printf("serial# %s, part# %s, rev# %s\n", frup[1], frup[2], frup[3]); 
	printf("MAC address %s\n", frup[0]);

	do {
		printf("\nEnter RLM serial number [%s]: ", 
					frup[1] != NULL?frup[1]:"123456");
		mygets(env_para[1], 16);
		if (env_para[1][0] == '\0' ){
			if ( frup[1] != NULL) {
				strcpy(env_para[1], frup[1]);
				break;
			}else {
				strcpy(env_para[1], "123456");
                break;
			}
		}
	}while (strlen(env_para[1]) != 6);

	do {
		printf("\nEnter RLM part number [%s]: ", 
				frup[2] != NULL?frup[2]:"111-11111");
    	mygets(env_para[2], 16);
		if (env_para[2][0] == '\0') {
			if ( frup[2] != NULL) {
            	strcpy(env_para[2], frup[2]);
				break;
			}else {
				strcpy(env_para[2], "111-11111");
                break;
			}
        }
	}while (strlen(env_para[2]) != 9 );

	do {
		printf("\nEnter RLM rev number [%s]: ", 
				frup[3] != NULL?frup[3]:"A0");
    	mygets(env_para[3], 16);
		if (env_para[3][0] == '\0') {
			if ( frup[3] != NULL) {
            	strcpy(env_para[3], frup[3]);
            	break;
			}else {
				strcpy(env_para[3], "A0");
                break;
			}
        }
	}while ( strlen(env_para[3]) != 2); 

	for (i = 0; i < 6; i++) {
		env_para[0][i*2] = frup[0][i*3];
		env_para[0][i*2+1] = frup[0][i*3+1];
	}
	env_para[0][12] = '\0';

	do {
		printf("\nEnter RLM MAC address [%s]: ", 
				frup[0]!=NULL?env_para[0]:"112233445566");
    	mygets(env_para[0], 20);
	}while ( strlen(env_para[0]) != 12 && env_para[0][0] != '\0');

	if (env_para[0][0] == '\0') {
		if (frup[0] != NULL) {
			strcpy(env_para[0], frup[0]);
		}else{
			strcpy(env_para[0], "11:22:33:44:55:66");
		}
	}else {
		 for ( i = 5; i >0 ; i--) {
            env_para[0][i*3 + 1] = env_para[0][i*2 + 1];
            env_para[0][i*3] = env_para[0][i*2];
            env_para[0][i*3 - 1] = ':';
        }
		env_para[0][17] = '\0';
    }

		
	printf("\nYou've entered the following information:\n");
    printf("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n");

    printf("RLM board: serial# %s, part# %s, rev# %s \n",
            env_para[1], env_para[2], env_para[3]);
    printf("           ethaddr %s\n\n", env_para[0]);
    printf("OK to save [y/n]? ");

	mygets(ch, 8);

	if ((ch[0] == 'y') || (ch[0] == 'Y')){
		puts("\n");
		setenv("serial#", env_para[1]);
    	setenv("part#", env_para[2]);
    	setenv("rev#", env_para[3]);
    	setenv("ethaddr", env_para[0]);

    	saveenv();
    } else {
        printf("\nSETUP CANCELLED!\n");
    }
	return 0;
}
		
U_BOOT_CMD(
        fvt, CFG_MAXARGS, 0, fvt,
        "fvt - program manufacture fru info\n",
        ""
        );

int check_ethaddr(void)
{
	char * pethaddr;
	pethaddr = getenv("ethaddr");
	if (!pethaddr) {
		printf("No ethaddr set\n");
		return 1;
	}
	return 0;
		 
}
