/*
 * (C) Copyright 2004 Network Appliance Inc. 
 *
 * Copyright 2005 Dan Gilson, Network Appliance, Inc.
 * Copyright 2005 Hong Huang, Network Appliance, Inc.
 */

/****************************************************************************
 * FLASH Memory Map as used by TQ Monitor:
 *
 *                          Start Address    Length
 * +-----------------------+ 0x4000_0000     Start of Flash -----------------
 * | MON8xx code           | 0x4000_0100     Reset Vector
 * +-----------------------+ 0x400?_????
 * | (unused)              |
 * +-----------------------+ 0x4001_FF00
 * | Ethernet Addresses    |                 0x78
 * +-----------------------+ 0x4001_FF78
 * | (Reserved for MON8xx) |                 0x44
 * +-----------------------+ 0x4001_FFBC
 * | Lock Address          |                 0x04
 * +-----------------------+ 0x4001_FFC0                     ^
 * | Hardware Information  |                 0x40            | MON8xx
 * +=======================+ 0x4002_0000 (sector border)    -----------------
 * | Autostart Header      |                                 | Applications
 * | ...                   |                                 v
 *
 *****************************************************************************/

typedef enum {
    LUNA_GPIO_TEST_MODE = 1,
    LUNA_GPIO_RESET_CPU,
    LUNA_GPIO_PRIMARY_IMG,
    LUNA_GPIO_STATUS_LED,
    LUNA_GPIO_POST_LED_8,
    LUNA_GPIO_BACKUP_IMG,
    LUNA_GPIO_WATCHDOG,
    LUNA_GPIO_RESERVED_1,
    LUNA_GPIO_RESERVED_2,
    LUNA_GPIO_RESERVED_3,
    LUNA_GPIO_POST_LED_1,
    LUNA_GPIO_POST_LED_7,
    LUNA_GPIO_POST_LED_6,
    LUNA_GPIO_POST_LED_5,
    LUNA_GPIO_POST_LED_4,
    LUNA_GPIO_POST_LED_3,
    LUNA_GPIO_IRQ_0,
    LUNA_GPIO_IRQ_1,
    LUNA_GPIO_IRQ_2,
    LUNA_GPIO_IRQ_3,
    LUNA_GPIO_FLASH_BUSY,
    LUNA_GPIO_POST_LED_2,
    LUNA_GPIO_RESERVED_4
} luna_gpio_t;

#define CPLD_PHYS_ADDR 0xF0000000

typedef enum {
	REBOOTED_VIA_POWER_ON = 0x00,
	REBOOTED_VIA_WATCHDOG = 0x80,
	REBOOTED_VIA_SW = 0x40,
	REBOOTED_VIA_APLNCE = 0x20,
	REBOOTED_VIA_UNKNOWN = 0xff
} luna_reboot_t;
 
typedef enum {
	EXEC_PRMRY_FW = 0x00,
	EXEC_BACKUP_FW = 0x10,
	EXEC_UNKNOWN_FW = 0xff
} luna_fw_img_t;

typedef enum {
    DATA_WALK = 0,
    ADDR_WALK = 1,
    STUCK_FAULT = 2,
    PATTERN_TEST = 3,
    WORD_WALK = 4,
    BYTE_WALK = 5,
    PARTIAL_WORD = 6,
    BYTE_PATTERN = 7,
    ECC_PATTERN = 8
}mem_test_suite_t;

#define ECC_EVEN_LANE0  0x10000000
#define ECC_EVEN_LANE1  0x20000000
#define ECC_EVEN_LANE2  0x40000000
#define ECC_EVEN_LANE3  0x80000000
#define ECC_ODD_LANE0   0x01000000
#define ECC_ODD_LANE1   0x02000000
#define ECC_ODD_LANE2   0x04000000
#define ECC_ODD_LANE3   0x08000000
#define ECC_CHECK_LOW   0x00400000
#define ECC_CHECK_HIGH  0x00800000
#define ECC_BAD_U10     ECC_EVEN_LANE0 | ECC_EVEN_LANE1 | ECC_ODD_LANE0 | ECC_ODD_LANE1
#define ECC_BAD_U9      ECC_EVEN_LANE2 | ECC_EVEN_LANE3 | ECC_ODD_LANE2 | ECC_ODD_LANE3
#define ECC_BAD_U8      ECC_CHECK_LOW | ECC_CHECK_HIGH
#define ECC_CORR        0x00200000
#define ECC_UNCORR      0x00100000

#define TRUE 1
#define FALSE 0

int gpio_init(void);
int luna_gpio_tristate(__u32 mask, __u32 data);
int luna_gpio_open_drain(__u32 mask, __u32 data);
int luna_gpio_in(__u32 mask, volatile __u32 *data);
int luna_gpio_out(__u32 mask, __u32 data);
int luna_post_led(__u8 post_code);
int luna_gpio_set(luna_gpio_t bit);
int luna_gpio_get(luna_gpio_t bit, volatile __u32 *val);
int luna_gpio_clr(luna_gpio_t bit);

