/*
 * (C) Copyright 2000
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 * Copyright 2005 Hong Huang, Network Appliance, Inc.
 */

#include <common.h>
#include "walnut405.h"
#include <asm/processor.h>
#include <spd_sdram.h>

int board_pre_init (void)
{
   /*-------------------------------------------------------------------------+
   | Interrupt controller setup for the Walnut board.
   | Note: IRQ 0-15  405GP internally generated; active high; level sensitive
   |       IRQ 16    405GP internally generated; active low; level sensitive
   |       IRQ 17-24 RESERVED
   |       IRQ 25 (EXT IRQ 0) FPGA; active high; level sensitive
   |       IRQ 26 (EXT IRQ 1) SMI; active high; level sensitive
   |       IRQ 27 (EXT IRQ 2) Not Used
   |       IRQ 28 (EXT IRQ 3) PCI SLOT 3; active low; level sensitive
   |       IRQ 29 (EXT IRQ 4) PCI SLOT 2; active low; level sensitive
   |       IRQ 30 (EXT IRQ 5) PCI SLOT 1; active low; level sensitive
   |       IRQ 31 (EXT IRQ 6) PCI SLOT 0; active low; level sensitive
   | Note for Walnut board:
   |       An interrupt taken for the FPGA (IRQ 25) indicates that either
   |       the Mouse, Keyboard, IRDA, or External Expansion caused the
   |       interrupt. The FPGA must be read to determine which device
   |       caused the interrupt. The default setting of the FPGA clears
   |
   +-------------------------------------------------------------------------*/

	mtdcr (uicsr, 0xFFFFFFFF);	/* clear all ints */
	mtdcr (uicer, 0x00000000);	/* disable all ints */
	mtdcr (uiccr, 0x00000020);	/* set all but FPGA SMI to be non-critical */
	mtdcr (uicpr, 0xFFFFFFE0);	/* set int polarities */
	mtdcr (uictr, 0x10000000);	/* set int trigger levels */
	mtdcr (uicvcr, 0x00000001);	/* set vect base=0,INT0 highest priority */
	mtdcr (uicsr, 0xFFFFFFFF);	/* clear all ints */

#define mtebc(reg, data)  mtdcr(ebccfga,reg);mtdcr(ebccfgd,data)
	/* BAS=0xF00,BS=0x0(1MB),BU=0x3(R/W),BW=0x0( 8 bits) */
	mtebc (pb1ap, 0x02815480);
	mtebc (pb1cr, 0xF0018000);

	/* BAS=0xF01,BS=0x0(1MB),BU=0x3(R/W),BW=0x0(8 bits) */
	mtebc (pb2ap, 0x04815A80);
	mtebc (pb2cr, 0xF0118000);

	/* BAS=0xF02,BS=0x0(1MB),BU=0x3(R/W),BW=0x0( 8 bits) */
	mtebc (pb3ap, 0x01815280);
	mtebc (pb3cr, 0xF0218000);

	/* BAS=0xF03,BS=0x0(1MB),BU=0x3(R/W),BW=0x0(8 bits) */
	mtebc (pb7ap, 0x01815280);
	mtebc (pb7cr, 0xF0318000);

	/* set UART1 control to select CTS/RTS */
#define FPGA_BRDC       0xF0300004
	*(volatile char *) (FPGA_BRDC) |= 0x1;

	return 0;
}


/* ------------------------------------------------------------------------- */

/*
 * Check Board Identity:
 */

int checkboard (void)
{
	unsigned char *s = getenv ("serial#");
	unsigned char *e;

	puts ("Board: ");

	if (!s || strncmp (s, "WALNUT405", 9)) {
		puts ("### No HW ID - assuming WALNUT405");
	} else {
		for (e = s; *e; ++e) {
			if (*e == ' ')
				break;
		}
		for (; s < e; ++s) {
			putc (*s);
		}
	}
	putc ('\n');

	return (0);
}


/* -------------------------------------------------------------------------
  initdram(int board_type) reads EEPROM via I2c. EEPROM contains all of
  the necessary info for SDRAM controller configuration
   ------------------------------------------------------------------------- */
long int initdram (int board_type)
{
	return  spd_sdram (0);
}

/* ------------------------------------------------------------------------- */

static int mem_data_test(void)
{
    uint * mem_start = CFG_SDRAM_BASE;
    uint * mem_test_start = (uint *)CFG_MEMTEST_START;
    uint * location;
    uint data;
    uint read_data;
	uint i;

	printf(".");
	location = mem_test_start;
    data = 1;
    for (i = 0; i <32; i++) {
        *location = data;
        *mem_start = 0xffffffff;
		read_data = *location;
        if (read_data != data) {
            printf("error ram test, walking 1\n");
			printf("location = %p, data expected = 0x%x, data read = 0x%x\n",location, data, read_data);
			return 1;
        }
        data <<= 1;
    /* need ecc check later */
    }  

    data = 0xfffffffe;
	location = mem_test_start;
    for (i = 0; i < 32; i++) {
        *location = data;
        *mem_start = 0;
		read_data = *location;
        if ( read_data != data) {
            printf("error ram test, walking 0\n");
			printf("location = %p, data expected = 0x%x, data read = 0x%x\n", 
					location, data, read_data);
			return 1;
        }
        data <<= 1;
    /* need ecc check later */
    }  

	return 0;
}

static int mem_addr_test(void)
{
	DECLARE_GLOBAL_DATA_PTR;
    /* TODO: XXX XXX XXX */
    uint mem_size = gd->ram_size;
    uint * mem_start = CFG_SDRAM_BASE;
    uint * location;
    int i;
    uint data;
    uint read_data;

	printf(".");
    data = 0x21426384;
    location = mem_start;
    i = 0x1;
    do {
        *location = data;
        location = mem_start + i;
        data += 0x01010101;
        i <<= 1;
    } while (i <= mem_size/4);

	location = mem_start;
    data = 0x21426384;
    i = 0x1;
    do {
        if ((read_data = *location ) != data) {
			printf("error ram test, address walking 1 test\n");
			printf("location = %p, data expected = 0x%x, data read = 0x%x\n", 
					location, data, read_data);
			return 1;
        }
        data += 0x01010101;
        location = mem_start + i;
        i <<= 1;
    }while ( i <= mem_size/4);

    location = mem_start;
    data = 0x17263544;
    i = 0x1;
    do {
        *location = data;
        location = mem_start + i;
        data += 0x04040404;
        i <<= 1;
    } while (i <= mem_size/4);

    location = mem_start;
    data = 0x17263544;
    i = 0x1;
    do {
        if ((read_data = *location ) != data) {
            printf("error ram test, address walking 1 test\n");
			printf("location = %p, data expected = 0x%x, data read = 0x%x\n",
                    location, data, read_data);
			return 1;
        }
        data += 0x04040404;
        location = mem_start + i;
        i <<= 1;
    }while ( i <= mem_size/4);

    location = mem_start + mem_size/4 -1;
    data = 0xa3b2c1d0;
    i = 0x1;
    do {
        *location = data;
        location = mem_start + mem_size/4 - 1 - i;
        data += 0x01010101;
        i <<= 1;
    } while ( i <= mem_size/4);

    location = mem_start + mem_size/4 - 1;
    data = 0xa3b2c1d0;
    i = 0x1;
    do {
        if ((read_data = *location) != data) {
            printf("error ram test, address walking 0 test\n");
			printf("location = %p, data expected = 0x%x, data read = 0x%x\n",
                    location, data, read_data);
			return 1;
        }
        data += 0x01010101;
        location = mem_start + mem_size/4 - 1 - i;
        i <<= 1;
    } while ( i <= mem_size/4);

	location = mem_start + mem_size/4 - 1;
    data = 0xb1a29384;
    i = 0x1;
    do {
        *location = data;
        location = mem_start + mem_size/4 - 1 - i;
        data += 0x02020202;
        i <<= 1;
    } while ( i <= mem_size/4);

    location = mem_start + mem_size/4 - 1;
    data = 0xb1a29384;
    i = 0x1;
    do {
        if ((read_data = *location) != data) {
            printf("error ram test, address walking 0 test\n");
			printf("location = %p, data expected = 0x%x, data read = 0x%x\n",
                    location, data, read_data);
			return 1;
        }
        data += 0x02020202;
        location = mem_start + mem_size/4 - 1 - i;
        i <<= 1;
    } while ( i <= mem_size/4);
	return 0;
}

static int mem_stuck_fault_test(void)
{
    DECLARE_GLOBAL_DATA_PTR;
    /* TODO: XXX XXX XXX */
    uint mem_size = gd->ram_size;
    uint * mem_start = CFG_SDRAM_BASE;
    uint * location;
    uint data;
    uint read_data;

	printf(".");
    location = mem_start;
	data = 0x00000000;
    while (location < ( mem_start + mem_size/4 )) {
        *location++ = data;
    }

    do {
		printf(".");
        location = mem_start;
        do {
            if ((read_data = *location ) != data) {
                printf("error memory test, stuck fault test\n");
				printf("location=%p, data expected=0x%x, data read=0x%x\n", 
						location, data, read_data);
				return 1;
            }
        }while (++location < (mem_start + mem_size/4));

        location = mem_start;
		data = 0xffffffff;

		printf(".");
        while (location < (mem_start + mem_size/4)) {
            *location++ = data;
        }

		printf(".");
        location = mem_start;
        do{
            if ((read_data = *location) != data) {
                printf("error memory test, stuck fault test\n");
				printf("location=%p, data expected=0x%x, data read=0x%x\n", 
						location, data, read_data);
				return 1;
            }
        }while (++location < (mem_start + mem_size/4));

		printf(".");
        location = mem_start;
		data = 0x00000000;
        while (location < (mem_start + mem_size/4)) {
            *location++ = data;
        }

		printf(".");
        location = mem_start;
        do {
            if ((read_data = *location ) != data) {
                printf("error memory test, stuck fault test\n");
				printf("location=%p, data expected=0x%x, data read=0x%x\n", 
                        location, data, read_data);
				return 1;
            }
        }while (++location < (mem_start + mem_size/4));
    }while (0);
	return 0;
}

uint pattern[]= {
	0xaaaaaaaa,
	0xcccccccc,
	0xf0f0f0f0,
	0xff00ff00,
	0xffff0000,
	0x0000ffff,
	0x00ff00ff,
	0x0f0f0f0f,
	0x33333333,
	0x55555555};

static int mem_data_pattern(void)
{
	uint * mem_test_start = (uint *)CFG_MEMTEST_START;
    uint * location;
	int num_patterns = sizeof(pattern)/sizeof(pattern[0]);
	uint data_read;
	uint i;

	location = mem_test_start;
	for (i = 0 ; i < num_patterns; i++) {
		*location = pattern[i];
		data_read = *location;
		if (data_read != pattern[i]) {
			printf("error memory test, pattern test\n");
			printf("location = %p, data expected = 0x%x, data read = 0x%x\n",
					location, pattern[i], data_read);
			return 1;
		}
	}
	return 0;
}		
		
int testdram (void)
{
	DECLARE_GLOBAL_DATA_PTR;
	/* TODO: XXX XXX XXX */
	uint rc;

	rc = mem_data_test();	

	if (rc != 0) {
		printf("dram test fail\n");
	}

	rc = mem_addr_test();

	if (rc != 0) {
        printf("dram test fail\n");
    }  

	rc = mem_stuck_fault_test();
	if (rc != 0) {
        printf("dram test fail\n");
    }  

	rc = mem_data_pattern();
	if (rc != 0) {
		printf("dram test fail\n");
	}

	printf("pass dram test\n");	
	
	return (0);
}

/* ------------------------------------------------------------------------- */
