/*
 * (C) Copyright 2000-2002
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * (C) Copyright 2001 Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Andreas Heppel <aheppel@sysgo.de>

 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 * Copyright 2005 Dan Gilson, Network Appliance, Inc.
 */

/* #define DEBUG */

#include <common.h>

#if defined(CFG_ENV_IS_IN_FLASH) /* Environment is in Flash */

#include <command.h>
#include <environment.h>
#include <linux/stddef.h>
#include <malloc.h>

#if ((CONFIG_COMMANDS&(CFG_CMD_ENV|CFG_CMD_FLASH)) == (CFG_CMD_ENV|CFG_CMD_FLASH))
#define CMD_SAVEENV
#elif defined(CFG_ENV_ADDR_REDUND)
#error Cannot use CFG_ENV_ADDR_REDUND without CFG_CMD_ENV & CFG_CMD_FLASH
#endif

#if defined(CFG_ENV_SIZE_REDUND) && (CFG_ENV_SIZE_REDUND < CFG_ENV_SIZE)
#error CFG_ENV_SIZE_REDUND should not be less then CFG_ENV_SIZE
#endif

#ifdef CONFIG_INFERNO
# ifdef CFG_ENV_ADDR_REDUND
#error CFG_ENV_ADDR_REDUND is not implemented for CONFIG_INFERNO
# endif
#endif

/* ERC_OK - Error Return Code OK (good status) */
#define ERC_OK 0
#define ERC_BAD -1

char * env_name_spec = "Flash";

#ifdef ENV_IS_EMBEDDED

extern uchar environment[];
env_t *env_ptr = (env_t *)(&environment[0]);

#ifdef CMD_SAVEENV
/* static env_t *flash_addr = (env_t *)(&environment[0]);-broken on ARM-wd-*/
static env_t *flash_addr = (env_t *)CFG_ENV_ADDR;
#endif

#else /* ! ENV_IS_EMBEDDED */

env_t *env_ptr = (env_t *)CFG_ENV_ADDR;
#ifdef CMD_SAVEENV
static env_t *flash_addr = (env_t *)CFG_ENV_ADDR;
#endif

#endif /* ENV_IS_EMBEDDED */

#ifdef CFG_ENV_ADDR_REDUND

#ifdef CMD_SAVEENV
int save_to_flash(env_t *flash_addr, ulong flash_end_addr);
#endif

static env_t *flash_addr_backup = (env_t *)CFG_ENV_ADDR_REDUND;

/* CFG_ENV_ADDR is supposed to be on sector boundary */
static ulong end_addr = CFG_ENV_ADDR + CFG_ENV_SECT_SIZE - 1;
static ulong end_addr_backup = CFG_ENV_ADDR_REDUND + CFG_ENV_SECT_SIZE - 1;

static uchar active_flag = 1;
static uchar obsolete_flag = 0;
#endif

extern uchar default_environment[];
extern int default_environment_size;


uchar env_get_char_spec (int index)
{
	DECLARE_GLOBAL_DATA_PTR;

	return ( *((uchar *)(gd->env_addr + index)) );
}

#ifdef CFG_ENV_ADDR_REDUND

int  env_init(void)
{
	DECLARE_GLOBAL_DATA_PTR;

	int crc_primary_ok =
		(crc32(0, flash_addr->data, ENV_SIZE) == flash_addr->crc);
	int crc_backup_ok =
		(crc32(0, flash_addr_backup->data, ENV_SIZE) == flash_addr_backup->crc);

	ulong addr_default = (ulong)&default_environment[0];
	ulong addr_primary = (ulong)&(flash_addr->data);
	ulong addr_backup = (ulong)&(flash_addr_backup->data);

	if (crc_primary_ok)
	{
		/* normal case */
		gd->env_addr  = addr_primary;
		gd->env_valid = 1;
	}
	else if (crc_backup_ok)
	{
		gd->env_addr  = addr_backup;
		gd->env_valid = 2;
	}
	else 
	{
		gd->env_addr  = addr_default;
		gd->env_valid = 0;
	}


	return (0);
}

#ifdef CMD_SAVEENV

char *strcpy_double_null( char * to, const char * from)
{
	char *save = to;
	assert(to != NULL);
	assert(from != NULL);

	for (; ! ( ((*to = *from) == '\0') && (*(from+1) == '\0')); ++to, ++from);
	*to = *from; /* 2nd NULL */
	return(save);
}

int save_to_flash(env_t *flash_addr, ulong flash_end_addr)
{
	char *saved_data = NULL;
	int rc = 1;
#if CFG_ENV_SECT_SIZE > CFG_ENV_SIZE
	ulong up_data = 0;
#endif
	DECLARE_GLOBAL_DATA_PTR;
	env_t env_src;
	
	strcpy_double_null(env_src.data,(char *)gd->env_addr);
	env_src.crc = crc32(0, env_src.data, ENV_SIZE);

	debug ("Protect off %08lX ... %08lX\n",
		(ulong)flash_addr, flash_end_addr);

	if (flash_sect_protect (0, (ulong)flash_addr, flash_end_addr)) {
		goto Done;
	}

#if CFG_ENV_SECT_SIZE > CFG_ENV_SIZE
	up_data = (flash_end_addr + 1 - ((long)flash_addr + CFG_ENV_SIZE));
	debug ("Data to save 0x%x\n", up_data);
	if (up_data) {
		if ((saved_data = malloc(up_data)) == NULL) {
			printf("Unable to save the rest of sector (%ld)\n",
				up_data);
			goto Done;
		}
		memcpy(saved_data,
			(void *)((long)flash_addr + CFG_ENV_SIZE), up_data);
		debug ("Data (start 0x%x, len 0x%x) saved at 0x%x\n",
			   (long)flash_addr + CFG_ENV_SIZE,
				up_data, saved_data);
	}
#endif
	puts ("Erasing Flash...");
	debug (" %08lX ... %08lX ...",
		(ulong)flash_addr, flash_end_addr);

	if (flash_sect_erase ((ulong)flash_addr, flash_end_addr)) {
		goto Done;
	}

	puts ("Writing to Flash... ");
	debug (" %08lX ... %08lX ...",
		(ulong)&(flash_addr->data),
		sizeof(env_src.data)+(ulong)&(flash_addr->data));
	if (flash_write(env_src.data,
			(ulong)&(flash_addr->data),
			sizeof(env_src.data)) )
	{
		debug ("flash_write: data:");
		flash_perror (rc);
		goto Done;
	}

	if (flash_write((char *)&(env_src.crc),
			(ulong)&(flash_addr->crc),
			sizeof(env_src.crc)) )
	{
		debug ("flash_write: crc:");
		flash_perror (rc);
		goto Done;
	}

	puts ("done\n");

#if CFG_ENV_SECT_SIZE > CFG_ENV_SIZE
	if (up_data) { /* restore the rest of sector */
		debug ("Restoring the rest of data to 0x%x len 0x%x\n",
			   (long)flash_addr + CFG_ENV_SIZE, up_data);
		if (flash_write(saved_data,
				(long)flash_addr + CFG_ENV_SIZE,
				up_data)) {
			flash_perror(rc);
			goto Done;
		}
	}
#endif

	rc = ERC_OK;
Done:

	if (saved_data)
		free (saved_data);
	/* try to re-protect */
	(void) flash_sect_protect (1, (ulong)flash_addr, flash_end_addr);

	return rc;
}

int saveenv(void)
{
/*
 * Save both images.
 * save the primary image first
 */
int rc = ERC_BAD;
	if ( (rc = save_to_flash(flash_addr, end_addr)) )
	{
		debug ("saveenv: saving primary returned: %d\n", rc);
		return rc;
	}
	if ( (rc =save_to_flash(flash_addr_backup, end_addr_backup)) )
	{
		debug ("saveenv: saving backup returned: %d\n", rc);
		return rc;
	}
	return ERC_OK;
}
#endif /* CMD_SAVEENV */

#else /* ! CFG_ENV_ADDR_REDUND */

int  env_init(void)
{
	DECLARE_GLOBAL_DATA_PTR;

	if (crc32(0, env_ptr->data, ENV_SIZE) == env_ptr->crc) {
		gd->env_addr  = (ulong)&(env_ptr->data);
		gd->env_valid = 1;
	} else {
		gd->env_addr  = (ulong)&default_environment[0];
		gd->env_valid = 0;
	}

	return (0);
}

#ifdef CMD_SAVEENV

int saveenv(void)
{
	int	len, rc;
	ulong	end_addr;
	ulong	flash_sect_addr;
#if defined(CFG_ENV_SECT_SIZE) && (CFG_ENV_SECT_SIZE > CFG_ENV_SIZE)
	ulong	flash_offset;
	uchar	env_buffer[CFG_ENV_SECT_SIZE];
#else
	uchar *env_buffer = (char *)env_ptr;
#endif	/* CFG_ENV_SECT_SIZE */
	int rcode = 0;

#if defined(CFG_ENV_SECT_SIZE) && (CFG_ENV_SECT_SIZE > CFG_ENV_SIZE)

	flash_offset    = ((ulong)flash_addr) & (CFG_ENV_SECT_SIZE-1);
	flash_sect_addr = ((ulong)flash_addr) & ~(CFG_ENV_SECT_SIZE-1);

	debug ( "copy old content: "
		"sect_addr: %08lX  env_addr: %08lX  offset: %08lX\n",
		flash_sect_addr, (ulong)flash_addr, flash_offset);

	/* copy old contents to temporary buffer */
	memcpy (env_buffer, (void *)flash_sect_addr, CFG_ENV_SECT_SIZE);

	/* copy current environment to temporary buffer */
	memcpy ((uchar *)((unsigned long)env_buffer + flash_offset),
		env_ptr,
		CFG_ENV_SIZE);

	len	 = CFG_ENV_SECT_SIZE;
#else
	flash_sect_addr = (ulong)flash_addr;
	len	 = CFG_ENV_SIZE;
#endif	/* CFG_ENV_SECT_SIZE */

#ifndef CONFIG_INFERNO
	end_addr = flash_sect_addr + len - 1;
#else
	/* this is the last sector, and the size is hardcoded here */
	/* otherwise we will get stack problems on loading 128 KB environment */
	end_addr = flash_sect_addr + 0x20000 - 1;
#endif

	debug ("Protect off %08lX ... %08lX\n",
		(ulong)flash_sect_addr, end_addr);

	if (flash_sect_protect (0, flash_sect_addr, end_addr))
		return 1;

	puts ("Erasing Flash...");
	if (flash_sect_erase (flash_sect_addr, end_addr))
		return 1;

	puts ("Writing to Flash... ");
	rc = flash_write(env_buffer, flash_sect_addr, len);
	if (rc != 0) {
		flash_perror (rc);
		rcode = 1;
	} else {
		puts ("done\n");
	}

	/* try to re-protect */
	(void) flash_sect_protect (1, flash_sect_addr, end_addr);
	return rcode;
}

#endif /* CMD_SAVEENV */

#endif /* CFG_ENV_ADDR_REDUND */

void env_relocate_spec (void)
{
#if !defined(ENV_IS_EMBEDDED) || defined(CFG_ENV_ADDR_REDUND)
#ifdef CFG_ENV_ADDR_REDUND
#ifdef CMD_SAVEENV

	DECLARE_GLOBAL_DATA_PTR;

	int crc_backup_ok =
		(crc32(0, flash_addr_backup->data, ENV_SIZE) == flash_addr_backup->crc);

	if ( gd->env_valid == 0) {
		puts ("*** Warning - some problems detected: reading environment;\n");
		puts ("*** Overwriting to primary and backup env flash area\n");
		if ( save_to_flash(flash_addr, end_addr))
			puts ("*** Saving primary env flash area failed!\n");
		if ( save_to_flash(flash_addr_backup, end_addr_backup))
			puts ("*** Saving backup env flash area failed!\n");
	}
		
	if ( gd->env_valid == 2) {
		puts ("*** Warning - some problems detected: reading environment;\n");
		puts ("*** Overwriting to primary image\n");
		if ( save_to_flash(flash_addr, end_addr))
			puts ("*** Saving primary env flash area failed!\n");
	}
	if (!crc_backup_ok) {
		puts ("*** Warning - some problems detected: reading environment;\n");
		puts ("*** Overwriting to backup image\n");
		if ( save_to_flash(flash_addr_backup, end_addr_backup))
			puts ("*** Saving backup env flash area failed!\n");
	}
#endif
	strcpy_double_null(env_ptr->data,(char *)gd->env_addr);
	env_ptr->crc = crc32(0, env_ptr->data, ENV_SIZE);
#else /* CFG_ENV_ADDR_REDUND */
	memcpy (env_ptr, (void*)flash_addr, CFG_ENV_SIZE);
#endif /* CFG_ENV_ADDR_REDUND */
#endif /* ! ENV_IS_EMBEDDED || CFG_ENV_ADDR_REDUND */
}

#endif /* CFG_ENV_IS_IN_FLASH */
